@@ -41,7 +41,6 @@ struct 	html {
 	struct hentryq	 	  headers_used; /* headers we've seen */
 	ssize_t			  headers_offs; /* header offset */
 	unsigned int 		  flags; /* "oflags" in lowdown_opts */
-	int			  noescape; /* don't escape text */
 	struct lowdown_buf	**foots; /* footnotes */
 	size_t			  footsz; /* footnotes size  */
 };
@@ -55,7 +54,6 @@ escape_html(struct lowdown_buf *ob, const char *source,
 	size_t length, const struct html *st)
 {
 
-	assert(st->noescape == 0);
 	return hesc_html(ob, source, length, 
 		st->flags & LOWDOWN_HTML_OWASP, 0,
 		st->flags & LOWDOWN_HTML_NUM_ENT);
@@ -69,9 +67,7 @@ escape_htmlb(struct lowdown_buf *ob,
 	const struct lowdown_buf *in, const struct html *st)
 {
 
-	return st->noescape ?
-		hbuf_putb(ob, in) :
-		escape_html(ob, in->data, in->size, st);
+	return escape_html(ob, in->data, in->size, st);
 }
 
 /*
@@ -84,7 +80,6 @@ escape_literal(struct lowdown_buf *ob,
 	const struct lowdown_buf *in, const struct html *st)
 {
 
-	assert(st->noescape == 0);
 	return hesc_html(ob, in->data, in->size, 
 		st->flags & LOWDOWN_HTML_OWASP, 1,
 		st->flags & LOWDOWN_HTML_NUM_ENT);
@@ -99,7 +94,6 @@ escape_href(struct lowdown_buf *ob, const struct lowdown_buf *in,
 	const struct html *st)
 {
 
-	assert(st->noescape == 0);
 	return hesc_href(ob, in->data, in->size);
 }
 
@@ -1024,12 +1018,16 @@ rndr_root(struct lowdown_buf *ob,
 
 /*
  * Split "val" into multiple strings delimited by two or more whitespace
- * characters, padding the output with "starttag" and "endtag".
+ * characters, padding the output with "starttag" and "endtag".  If
+ * "href" is set, escape as if this were a link; if "attr" were set,
+ * escape as if this were an HTML attribute; if neither, then escape as
+ * if regular HTML.
  * Return zero on failure, non-zero on success.
  */
 static int
-rndr_meta_multi(struct lowdown_buf *ob, const char *b, int href,
-    const char *starttag, const char *endtag)
+rndr_meta_multi(const struct html *st, struct lowdown_buf *ob,
+    const char *b, int href, int attr, const char *starttag,
+    const char *endtag)
 {
 	const char	*start;
 	size_t		 sz, i, bsz;
@@ -1058,10 +1056,14 @@ rndr_meta_multi(struct lowdown_buf *ob, const char *b, int href,
 
 		if (!hbuf_puts(ob, starttag))
 			return 0;
-		if (!href && !hesc_attr(ob, start, sz))
+		if (attr && !hesc_attr(ob, start, sz))
 			return 0;
 		else if (href && !hesc_href(ob, start, sz))
 			return 0;
+		else if (!attr && !href && !hesc_html(ob, start, sz,
+		    st->flags & LOWDOWN_HTML_OWASP, 0,
+		    st->flags & LOWDOWN_HTML_NUM_ENT))
+			return 0;
 		if (!hbuf_puts(ob, endtag))
 			return 0;
 		if (!HBUF_PUTSL(ob, "\n"))
@@ -1076,36 +1078,22 @@ rndr_meta_multi(struct lowdown_buf *ob, const char *b, int href,
  * Return zero on failure, non-zero on success.
  */
 static int
-rndr_meta(struct lowdown_buf *ob,
-	const struct lowdown_buf *content,
-	struct lowdown_metaq *mq,
-	const struct lowdown_node *n, struct html *st)
+rndr_meta(struct html *st, const struct lowdown_node *n, 
+    struct lowdown_metaq *mq)
 {
 	struct lowdown_meta	*m;
 	ssize_t			 val;
 	const char		*ep;
 
-	m = calloc(1, sizeof(struct lowdown_meta));
-	if (m == NULL)
-		return 0;
-	TAILQ_INSERT_TAIL(mq, m, entries);
-
-	m->key = strndup(n->rndr_meta.key.data,
-		n->rndr_meta.key.size);
-	if (m->key == NULL)
-		return 0;
-	m->value = strndup(content->data, content->size);
-	if (m->value == NULL)
+	if ((m = lowdown_get_meta(n, mq)) == NULL)
 		return 0;
 
 	if (strcmp(m->key, "shiftheadinglevelby") == 0) {
-		val = (ssize_t)strtonum
-			(m->value, -100, 100, &ep);
+		val = (ssize_t)strtonum(m->value, -100, 100, &ep);
 		if (ep == NULL)
 			st->headers_offs = val + 1;
 	} else if (strcmp(m->key, "baseheaderlevel") == 0) {
-		val = (ssize_t)strtonum
-			(m->value, 1, 100, &ep);
+		val = (ssize_t)strtonum(m->value, 1, 100, &ep);
 		if (ep == NULL)
 			st->headers_offs = val;
 	}
@@ -1124,7 +1112,7 @@ rndr_doc_header(struct lowdown_buf *ob,
 					*affil = NULL, *date = NULL,
 					*copy = NULL, *rcsauthor = NULL, 
 					*rcsdate = NULL, *css = NULL,
-					*script = NULL;
+					*script = NULL, *header = NULL;
 
 	TAILQ_FOREACH(m, mq, entries)
 		if (strcasecmp(m->key, "author") == 0)
@@ -1145,6 +1133,8 @@ rndr_doc_header(struct lowdown_buf *ob,
 			css = m->value;
 		else if (strcasecmp(m->key, "javascript") == 0)
 			script = m->value;
+		else if (strcasecmp(m->key, "htmlheader") == 0)
+			header = m->value;
 
 	/* Overrides. */
 
@@ -1165,13 +1155,13 @@ rndr_doc_header(struct lowdown_buf *ob,
 		    "<meta name=\"viewport\" content=\""
 		    "width=device-width,initial-scale=1\" />\n"))
 			return 0;
-		if (!rndr_meta_multi(ob, affil, 0,
+		if (!rndr_meta_multi(st, ob, affil, 0, 1,
 		    "<meta name=\"creator\" content=\"", "\" />"))
 			return 0;
-		if (!rndr_meta_multi(ob, author, 0,
+		if (!rndr_meta_multi(st, ob, author, 0, 1,
 		    "<meta name=\"author\" content=\"", "\" />"))
 			return 0;
-		if (!rndr_meta_multi(ob, copy, 0,
+		if (!rndr_meta_multi(st, ob, copy, 0, 1,
 		    "<meta name=\"copyright\" content=\"", "\" />"))
 			return 0;
 		if (date != NULL) {
@@ -1201,10 +1191,10 @@ rndr_doc_header(struct lowdown_buf *ob,
 			if (!HBUF_PUTSL(ob, "\" />\n"))
 				return 0;
 		}
-		if (!rndr_meta_multi(ob, css, 1,
+		if (!rndr_meta_multi(st, ob, css, 1, 0,
 		    "<link rel=\"stylesheet\" href=\"", "\" />"))
 			return 0;
-		if (!rndr_meta_multi(ob, script, 1,
+		if (!rndr_meta_multi(st, ob, script, 1, 0,
 		     "<script src=\"", "\"></script>"))
 			return 0;
 
@@ -1217,7 +1207,19 @@ rndr_doc_header(struct lowdown_buf *ob,
 			    st->flags & LOWDOWN_HTML_OWASP, 0,
 			    st->flags & LOWDOWN_HTML_NUM_ENT))
 			return 0;
-		if (!HBUF_PUTSL(ob, "</title>\n</head>\n<body>\n"))
+		if (!HBUF_PUTSL(ob, "</title>\n"))
+			return 0;
+
+		/* Optional raw HTML header. */
+
+		if (header != NULL) {
+			if (!hbuf_puts(ob, header))
+				return 0;
+			if (header[strlen(header) - 1] != '\n' &&
+			    !HBUF_PUTSL(ob, "\n"))
+				return 0;
+		}
+		if (!HBUF_PUTSL(ob, "</head>\n<body>\n"))
 			return 0;
 	}
 
@@ -1236,7 +1238,7 @@ rndr_doc_header(struct lowdown_buf *ob,
 		     !HBUF_PUTSL(ob, "</h1>\n")))
 				return 0;
 		if (author != NULL &&
-		    !rndr_meta_multi(ob, author, 0,
+		    !rndr_meta_multi(st, ob, author, 0, 0,
 			    "<p class=\"author\">", "</p>"))
 			return 0;
 		if (date != NULL &&
@@ -1264,16 +1266,6 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq, struct html *st,
 	if ((tmp = hbuf_new(64)) == NULL)
 		return 0;
 
-	/*
-	 * If we're processing metadata, don't escape the content as we
-	 * read and parse it.  This prevents double-escaping.  We'll
-	 * properly escape things as we inline them (standalone mode) or
-	 * when we write body text.
-	 */
-
-	if (n->type == LOWDOWN_META)
-		st->noescape = 1;
-
 	TAILQ_FOREACH(child, &n->children, entries)
 		if (!rndr(tmp, mq, st, child))
 			goto out;
@@ -1313,9 +1305,8 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq, struct html *st,
 		rc = rndr_doc_header(ob, tmp, mq, st);
 		break;
 	case LOWDOWN_META:
-		st->noescape = 0;
 		if (n->chng != LOWDOWN_CHNG_DELETE)
-			rc = rndr_meta(ob, tmp, mq, n, st);
+			rc = rndr_meta(st, n, mq);
 		break;
 	case LOWDOWN_HEADER:
 		rc = rndr_header(ob, tmp, n, st);
