@@ -53,6 +53,7 @@ struct term {
 	size_t			  footsz; /* footnotes size  */
 	int			  footoff; /* don't collect (tables) */
 	struct lowdown_metaq	  metaq; /* metadata */
+	const struct lowdown_node*in_link; /* in an OSC8 hyperlink */
 };
 
 /*
@@ -181,15 +182,17 @@ static ssize_t
 rndr_escape(struct term *term, struct lowdown_buf *out,
 	const char *buf, size_t sz)
 {
-	size_t	 i, start = 0, cols = 0;
-	ssize_t	 ret;
+	size_t		 i, start = 0, cols = 0;
+	ssize_t		 ret;
+	unsigned char	 ch;
 
 	/* Don't allow control characters through. */
 
-	for (i = 0; i < sz; i++)
-		if (iscntrl((unsigned char)buf[i])) {
-			ret = rndr_mbswidth
-				(term, buf + start, i - start);
+	for (i = 0; i < sz; i++) {
+		ch = (unsigned char)buf[i];
+		if (ch < 0x80 && iscntrl(ch)) {
+			ret = rndr_mbswidth (term, buf + start,
+				i - start);
 			if (ret < 0)
 				return -1;
 			cols += ret;
@@ -197,6 +200,7 @@ rndr_escape(struct term *term, struct lowdown_buf *out,
 				return -1;
 			start = i + 1;
 		}
+	}
 
 	/* Remaining bytes. */
 
@@ -243,6 +247,58 @@ rndr_buf_unstyle(const struct term *term,
 	return HBUF_PUTSL(out, "\033[0m");
 }
 
+/*
+ * Start sequence for a terminal link (only in ANSI mode).  
+ */
+static int
+rndr_buf_osc8_open(const struct term *term, struct lowdown_buf *out,
+    const struct lowdown_node *n)
+{
+	const struct lowdown_buf	*uri = NULL;
+
+	if (term->opts & LOWDOWN_TERM_NOANSI)
+		return 1;
+	
+	if (n->type == LOWDOWN_LINK_AUTO)
+		uri = &n->rndr_autolink.link;
+	else if (n->type == LOWDOWN_LINK)
+		uri = &n->rndr_link.link;
+	else if (n->type == LOWDOWN_IMAGE)
+		uri = &n->rndr_image.link;
+
+	/*
+	 * Don't output an id for the link.  It's trivial to have a
+	 * random per-page value for this identifier (e.g., to
+	 * initialise a random number then append the node identifier),
+	 * but let the terminal handle this.
+	 */
+
+	assert(uri != NULL);
+	return HBUF_PUTSL(out, "\033]8;;") &&
+	    hbuf_putb(out, uri) &&
+	    HBUF_PUTSL(out, "\033\\");
+}
+
+/*
+ * Close a currently-open link.
+ */
+static int
+rndr_buf_osc8_close(const struct term *term, struct lowdown_buf *out)
+{
+
+	if (term->opts & LOWDOWN_TERM_NOANSI)
+		return 1;
+
+	/*
+	 * It would be trivial to crawl up our parent chain and either
+	 * switch from the current link or close out the link context
+	 * entirely, but lowdown(5) stipulates that nested links are not
+	 * possible.
+	 */
+
+	return HBUF_PUTSL(out, "\033]8;;\033\\");
+}
+
 /*
  * Output style "s" into "out" as an ANSI escape.  If "s" does not have
  * any style information or is NULL, output nothing.  Return zero on
@@ -408,7 +464,13 @@ static int
 rndr_buf_endwords(struct term *term, struct lowdown_buf *out,
 	const struct lowdown_node *n, const struct sty *osty)
 {
+	/*
+	 * If an OSC8 hyperlink should be closed, do it now (it doesn't
+	 * matter where this appears in relation to other styling).
+	 */
 
+	if (term->in_link && !rndr_buf_osc8_close(term, out))
+		return 0;
 	if (rndr_buf_endstyle(n))
 		return rndr_buf_unstyle(term, out, NULL);
 	if (osty != NULL)
@@ -682,9 +744,11 @@ rndr_buf_startline_prefixes(struct term *term,
 }
 
 /*
- * Like rndr_buf_startwords(), but at the start of a line.
- * This also outputs all line prefixes of the block context.
- * Return zero on failure (memory), non-zero on success.
+ * Like rndr_buf_startwords(), but at the start of a line.  (Unlike
+ * rndr_buf_endline(), which calls rndr_buf_endwords(), this does not
+ * call rndr_buf_startwords().)  This also outputs all line prefixes of
+ * the block context.  Return zero on failure (memory), non-zero on
+ * success.
  */
 static int
 rndr_buf_startline(struct term *term, struct lowdown_buf *out,
@@ -699,6 +763,16 @@ rndr_buf_startline(struct term *term, struct lowdown_buf *out,
 	memset(&s, 0, sizeof(struct sty));
 	if (!rndr_buf_startline_prefixes(term, &s, n, out, &depth))
 		return 0;
+
+	/*
+	 * If an OSC8 hyperlink should be printed, do it now (it doesn't
+	 * matter where this appears in relation to other styling).
+	 */
+
+	if (term->in_link != NULL &&
+	    !rndr_buf_osc8_open(term, out, term->in_link))
+		return 0;
+
 	if (osty != NULL)
 		rndr_node_style_apply(&s, osty);
 	return rndr_buf_style(term, out, &s);
@@ -764,6 +838,15 @@ rndr_buf_startwords(struct term *term, struct lowdown_buf *out,
 {
 	struct sty	 s;
 
+	/*
+	 * If an OSC8 hyperlink should be printed, do it now (it doesn't
+	 * matter where this appears in relation to other styling).
+	 */
+
+	if (term->in_link != NULL &&
+	    !rndr_buf_osc8_open(term, out, term->in_link))
+		return 0;
+
 	assert(!term->last_blank);
 	assert(term->col > 0);
 
@@ -1270,62 +1353,6 @@ rndr_doc_header(struct lowdown_buf *ob, struct term *st,
 	return 1;
 }
 
-/*
- * Extract metadata into "metaq".  This avoids calling rndr_buf()
- * because of all the line fiddling: because we know the contents of the
- * LOWDOWN_META, serialise directly into the metadata value without
- * formatting.
- *
- * This will be serialised to the ouptut buffer in rndr_doc_header().
- *
- * Return zero on failure (memory), non-zero otherwise.
- */
-static int
-rndr_meta(struct term *st, const struct lowdown_node *n)
-{
-	struct lowdown_meta		*m;
-	const struct lowdown_node	*child;
-	struct lowdown_buf		*metatmp;
-	int32_t				 entity;
-
-	m = calloc(1, sizeof(struct lowdown_meta));
-	if (m == NULL)
-		return 0;
-	TAILQ_INSERT_TAIL(&st->metaq, m, entries);
-	m->key = strndup(n->rndr_meta.key.data,
-		n->rndr_meta.key.size);
-	if (m->key == NULL)
-		return 0;
-	if ((metatmp = hbuf_new(128)) == NULL)
-		return 0;
-	TAILQ_FOREACH(child, &n->children, entries) {
-		switch (child->type) {
-		case LOWDOWN_NORMAL_TEXT:
-			if (!hbuf_putb(metatmp,
-			    &child->rndr_normal_text.text))
-				return 0;
-			break;
-		case LOWDOWN_ENTITY:
-			entity = entity_find_iso
-				(&child->rndr_entity.text);
-			if (entity == 0)
-				break;
-			hbuf_truncate(st->tmp);
-			if (!rndr_entity(st->tmp, entity) ||
-			    !hbuf_putb(metatmp, st->tmp))
-				return 0;
-			break;
-		default:
-			abort();
-		}
-	}
-	m->value = strndup(metatmp->data, metatmp->size);
-	if (m->value == NULL)
-		return 0;
-	hbuf_free(metatmp);
-	return 1;
-}
-
 static int
 rndr(struct lowdown_buf *ob, struct term *st,
 	const struct lowdown_node *n)
@@ -1403,6 +1430,11 @@ rndr(struct lowdown_buf *ob, struct term *st,
 	/* Output leading content. */
 
 	switch (n->type) {
+	case LOWDOWN_IMAGE:
+	case LOWDOWN_LINK:
+	case LOWDOWN_LINK_AUTO:
+		st->in_link = n;
+		break;
 	case LOWDOWN_SUPERSCRIPT:
 		hbuf_truncate(st->tmp);
 		if (!hbuf_puts(st->tmp, ifx_super) ||
@@ -1447,7 +1479,7 @@ rndr(struct lowdown_buf *ob, struct term *st,
 			return 0;
 		break;
 	case LOWDOWN_META:
-		if (!rndr_meta(st, n))
+		if (lowdown_get_meta(n, &st->metaq) == NULL)
 			return 0;
 		break;
 	default:
@@ -1534,9 +1566,8 @@ rndr(struct lowdown_buf *ob, struct term *st,
 		if (st->opts & LOWDOWN_TERM_NOLINK)
 			break;
 		hbuf_truncate(st->tmp);
-		if (!HBUF_PUTSL(st->tmp, " "))
-			return 0;
-		if (!rndr_buf(st, ob, n, st->tmp, NULL))
+		if (!HBUF_PUTSL(st->tmp, " ") ||
+		    !rndr_buf(st, ob, n, st->tmp, NULL))
 			return 0;
 		if (st->opts & LOWDOWN_TERM_SHORTLINK) {
 			hbuf_truncate(st->tmp);
@@ -1604,9 +1635,14 @@ rndr(struct lowdown_buf *ob, struct term *st,
 		break;
 	}
 
-	/* Trailing block spaces. */
-
-	if (n->type == LOWDOWN_ROOT) {
+	switch (n->type) {
+	case LOWDOWN_IMAGE:
+	case LOWDOWN_LINK:
+	case LOWDOWN_LINK_AUTO:
+		st->in_link = NULL;
+		break;
+	case LOWDOWN_ROOT:
+		/* Trailing block spaces. */
 		if (st->footsz) {
 			if (!rndr_buf_vspace(st, ob, n, 2))
 				return 0;
@@ -1637,6 +1673,9 @@ rndr(struct lowdown_buf *ob, struct term *st,
 		for (i = 0; i < st->vmargin; i++)
 			if (!HBUF_PUTSL(ob, "\n"))
 				return 0;
+		break;
+	default:
+		break;
 	}
 
 	return 1;
@@ -1651,6 +1690,7 @@ lowdown_term_rndr(struct lowdown_buf *ob,
 
 	TAILQ_INIT(&st->metaq);
 	st->stackpos = 0;
+	st->in_link = NULL;
 	rc = rndr(ob, st, n);
 	rndr_free_footnotes(st);
 	lowdown_metaq_free(&st->metaq);
